/***************************************************************************
Copyright (c) 2006 Peter Korzuszek, Damian Rakowski, KateOS Project
 
Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:
 
*Redistributions of source code must retain the above copyright notice,
this list of conditions and the following disclaimer.
 
*Redistributions in binary form must reproduce the above copyright notice,
this list of conditions and the following disclaimer in the documentation
and/or other materials provided with the distribution.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS
IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
DAMAGE. 
****************************************************************************/

#ifndef SMART_TOOLS_H
#define SMART_TOOLS_H

#ifndef ST_DEFAULT_SHELL
#define ST_DEFAULT_SHELL "/bin/sh"
#endif

#ifndef ST_DEFAULT_TEMP
#define ST_DEFAULT_TEMP "/tmp"
#endif

// number of tries to create unique file
#ifndef ST_FILE_CREATE_LIMIT
#define ST_FILE_CREATE_LIMIT 100
#endif

// where can I find tar application
#ifndef ST_TAR_PATH
#define ST_TAR_PATH "/bin/tar"
#endif

// where can I find file application
#ifndef ST_FILE_PATH
#define ST_FILE_PATH "/usr/bin/file"
#endif

// maximum number of arguments
#ifndef ST_MAX_ARGS
#define ST_MAX_ARGS 32768
#endif

namespace SmartTools
{
	const char endl = '\n';
	
	enum Position{ Left, Right, Top, Bottom };
	
	enum Color { White, LightRed, DarkRed, LightGreen, Blue,
		    Green, Cyan, Purple, Brown, LightGray, DarkGray, LightBlue,
		    LightCyan, LightPurple, Yellow, Default };
		    
	enum COutputType { Out_Normal, Out_Error };	    
		
	enum OpenFileMode{
	
	Read, ReadWrite,
	// positions below creates file if don't exist 
	EraseWrite, EraseReadWrite,
	// positions below sets stream position at end of file 
	Append, AppendRead
	
	};
	
	enum OptionValues { None, Single, Multiple };
	
	enum ArchiveType { ARCHIVE_TAR, ARCHIVE_TAR_GZ, ARCHIVE_TAR_BZ2, ARCHIVE_AUTO };
			    
}

/*!
	\mainpage SmartTools Library Documentation
	
	\section intro_sec Introduction
	
	The SmartTools library is a collection of some very useful classes for programming
	on Unix-like systems and is optimized for max simplicity. It provides its
	own string class called StString, which is faster and more useful than std::string;
	it also includes input/output classes like StOutput, StFile or StDir. In addition,
	it provides the following classes:
	
	<ul>
		<li>StArgs - manage your application's runtime arguments</li>
		<li>StApplication - run any application, and then read the output</li>
		<li>StConfigFile - manage your own config files</li>
		<li>StFs - create / move / delete files or directories</li>
		<li>StMd5 - do you need the checksum?</li>
		<li>StPermission - manage file permissions</li>
		<li>StRandom() - really random numbers</li>
		<li>StStringList - manipulate lists of strings (can be text files)</li>
		<li>StTemp - easily manage temp directory</li>
		<li>StThread - easy threads</li>
		<li>StTime - get, create and compare times</li>
		<li>StUrl - download something from the net</li>
		<li>StValueList - keep objects in vector-like lists</li>
	</ul>
	
	SmartTools is a well-used library in the KateOS Linux distribution's system
	administration applications.
	
	\section download_sec Download
	
	The only way at this time to download SmartTools is through svn. Be aware that
	svn version is unstable most of time and contains known bugs. To download make sure
	that you have the subversion client installed on your system, then type in bash:
	
	svn checkout svn://pragma.pl:1310/kateos
	
	This will create ./kateos directory.
	
	\section install_sec Installation
	
	Simply type this:
	
	cd kateos/libsmarttools<br>
	./configure<br>
	make<br>
	su<br>
	make install<br>
	
	Make sure that you don't have libsmarttools.so* libraries in your system already.
	If this library was installed by a package manager then try to remove this package
	or delete the library files manualy if the library is a part of another package.
*/

#endif //SMART_TOOLS_H

