/***************************************************************************
Copyright (c) 2006 Peter Korzuszek, Damian Rakowski, Mateusz Bilski,
KateOS Project
 
Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:
 
*Redistributions of source code must retain the above copyright notice,
this list of conditions and the following disclaimer.
 
*Redistributions in binary form must reproduce the above copyright notice,
this list of conditions and the following disclaimer in the documentation
and/or other materials provided with the distribution.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS
IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
DAMAGE. 
****************************************************************************/

#ifndef SMART_TOOLS_ARGS_H
#define SMART_TOOLS_ARGS_H

#include "stdir.h"
#include "ststring.h"
#include "smarttools.h"
#include "ststringlist.h"

const int ER_STARCHIVE_TYPE_AUTO = 1;
const int ER_STARCHIVE_FILE = 2;

/*!
	\brief This class allows for basic archive operations.
	
	You can get the list, pack, and unpack gzipped, bzipped, or uncompressed tarballs.
*/
class StArchive
{
	
	public:
		StArchive() { };
		~StArchive() { };
		
		/*!
			\return list of files
			\note If the type is not given, the type will be identified automatically.
		*/
		static StStringList list( StFile packageFile, SmartTools::ArchiveType type=SmartTools::ARCHIVE_AUTO );
	
		/*!
			This function creates an archive. It checks if files exist. If they don't, it adds an error
			to sterror and exits.
			\note If the type is not given, the type will be guessed from the filename.
		*/
		static bool pack( const StString& fileName, const StValueList<StFile>& fileList, SmartTools::ArchiveType type=SmartTools::ARCHIVE_AUTO );
		
		/*!
			Unpack archive to outputDirectory. If the file list is not empty, unpack only given files.
			\note If the type is not given, the type will be guessed.
		*/
		static bool unpack( StFile packageFile, StDir outputDirectory, SmartTools::ArchiveType type=SmartTools::ARCHIVE_AUTO, const StStringList& fileList = StStringList() );
	private:
		static SmartTools::ArchiveType getType( StFile packageFile );
};

#endif //SMART_TOOLS_ARGS_H

