/***************************************************************************
Copyright (c) 2006 Peter Korzuszek, Damian Rakowski, KateOS Project
 
Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:
 
*Redistributions of source code must retain the above copyright notice,
this list of conditions and the following disclaimer.
 
*Redistributions in binary form must reproduce the above copyright notice,
this list of conditions and the following disclaimer in the documentation
and/or other materials provided with the distribution.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS
IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
DAMAGE. 
****************************************************************************/

#ifndef SMART_TOOLS_CONFIG_FILE_H
#define SMART_TOOLS_CONFIG_FILE_H

#include "stfile.h"
#include "ststring.h"
#include "stvaluelist.h"

/*!
	\brief This class allows creation and management of config files.
	
	You can load a config file, change or add sections or key values, and save it.
*/

class StConfigFile
{
	public:
		StConfigFile(){}
		
		/** Load a file to the list. */
		bool load( const StFile& file );
		
		/** Save a list to the file. */
		bool save( const StFile& file );
				
		/** 
			\param section the section which includes the key
			\param key the key whose value you want to read.
			\return value of the key
		*/
		StString get( const StString& section, const StString& key );
		
		/**
			Set the key's value in a given section to the list.
			\param section the section where you want to set the key's value.
			\param key the name of the key.
			\param value the key's value.
		*/
		void set( const StString& section, const StString& key, const StString& value );
	
	private:
		typedef struct{
			StString name;
			StString value;
		} Key;
		
		typedef struct{
			StString name;
			StValueList<Key> keys;
		} Section;
		
		/** List of sections which include list of keys. */
		StValueList<Section> sections;
		
		int findSection( const StString& name );
		int findKey( const StString& name, int sectionIndex );
};

#endif //SMART_TOOLS_CONFIG_FILE_H
