/***************************************************************************
Copyright (c) 2006 Peter Korzuszek, Damian Rakowski, KateOS Project
 
Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:
 
*Redistributions of source code must retain the above copyright notice,
this list of conditions and the following disclaimer.
 
*Redistributions in binary form must reproduce the above copyright notice,
this list of conditions and the following disclaimer in the documentation
and/or other materials provided with the distribution.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS
IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
DAMAGE. 
****************************************************************************/

#ifndef SMART_TOOLS_URL_H
#define SMART_TOOLS_URL_H

#include <curl/curl.h>

#include "stfile.h"
#include "ststring.h"
#include "stthread.h"
#include "stoutput.h"

/*!
	\brief StUrl enables to download files.
	
	It allows you to download file from remote url or local path and
	set the low speed and connection timeout.
*/

class StUrl : public StThread
{
	public:
		StUrl(){ init();  bytesDone = bytesTotal = 0; }
		StUrl( const StUrl& url );
		~StUrl(){ curl_easy_cleanup(cHandle); }
		
		/*!
			Returns true when download starts
		*/
		bool download( bool append=false );
		
		void setRemoteUrl( const StString& url ){ remote = url; }
		void setLocalPath( const StString& path ){ local = path; }
		
		void setLowSpeedTimeout( long seconds );
		long getLowSpeedTimeout() const { return lowSpeedTimeout; }
		
		void setConnectionTimeout( long seconds );
		long getConnectionTimeout() const { return connectionTimeout; }
		
		StString getRemoteUrl(){ return remote; }
		StString getLocalPath(){ return local; }
		
		/** \return Stfile from given local path. */
		StFile getLocalFile(){ return StFile(getLocalPath()); }
		
		bool isDownloading() const { return !hasFinished(); }
		bool downloadSucceed() const { return succeed; }
		bool hasTimeout() const { return timeout; }
		
		unsigned getBytesDone(){ return bytesDone; }
		unsigned getBytesTotal(){ return bytesTotal; }
		
		/*! \return CURLcode that curl_easy_perform() returns */
		CURLcode getCurlReturnCode(){ return returnCode; }
	
	protected:
		/** Create new thread and download file. */
		virtual void run();
	
	private:
		CURL* cHandle;
		StString remote, local;
		StFile file;
		volatile bool succeed;
		volatile bool timeout;
		CURLcode returnCode;
		long lowSpeedTimeout; // how long it will last when transfer stopss
		long connectionTimeout;
		
		unsigned bytesDone, bytesTotal;
		unsigned bytesStartSize; // size of file before download
		
		void init();
		
		static size_t writeData( void* buffer, size_t size, size_t count, void* stFilePointer );
		static int setProgress( StUrl* url, double t, double d, double, double );
};

#endif //SMART_TOOLS_URL_H
